if ( $PSVersionTable.PSVersion.Major -lt 3) {
    $bindingFlags = [Reflection.BindingFlags] Instance,NonPublic,GetField
    $objectRef = $host.GetType().GetField(externalHostRef, $bindingFlags).GetValue($host)

    $bindingFlags = [Reflection.BindingFlags] Instance,NonPublic,GetProperty
    $consoleHost = $objectRef.GetType().GetProperty(Value, $bindingFlags).GetValue($objectRef, @())

    [void] $consoleHost.GetType().GetProperty(IsStandardOutputRedirected, $bindingFlags).GetValue($consoleHost, @())
    $bindingFlags = [Reflection.BindingFlags] Instance,NonPublic,GetField
    $field = $consoleHost.GetType().GetField(standardOutputWriter, $bindingFlags)
    $field.SetValue($consoleHost, [Console]::Out)
    $field2 = $consoleHost.GetType().GetField(standardErrorWriter, $bindingFlags)
    $field2.SetValue($consoleHost, [Console]::Out)
}
try {
    $title = "   VipNet CSP"
    $ok = "Ok"
    $import_dir = "./pfx_import"
    $my_arg = ""
    $store_path = $Env:LOCALAPPDATA + "\Crypto Pro\stores\my.sto"
    if ($args -contains "-my") {
        $my_arg = "-my"
    }
    [System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms")
    [System.Windows.Forms.MessageBox]::Show(",     .", "!", $ok, "Information")

    if (!(Test-Path $import_dir)) {
        New-Item -Path $import_dir -ItemType Directory
    }

    ./export_vipnet_containers.exe -outdir  $import_dir  $my_arg
    if (!$?) {
        $error_msg = "   ."
        $msg_type = "Stop"
        if ($lastexitcode -eq 2) {
            $error_msg = "    ."
            $msg_type = "Information"
        }
        if ($lastexitcode -eq 0x80090019) {
            $error_msg += "`r`n   VipNet CSP."
        }
        [System.Windows.Forms.MessageBox]::Show($error_msg, $title, $ok, $msg_type)
        Exit
    }

    $created_new = 0
    Get-ChildItem $import_dir -Filter *.pfx | select FullName |
    Foreach-Object {
        ./certmgr.exe -use_cp_capi -inst -trace 0 -pfx -keep_exportable -autodist -silent -pin 1 -file $_.FullName
        if ($lastexitcode -eq 0) {
            $created_new = 1    
        }
        Remove-Item $_.FullName
    }

    if ($created_new -ne 1) {
        if (Test-Path $store_path) {
            Remove-Item $store_path
        }
        [System.Windows.Forms.MessageBox]::Show("    ", $title, $ok, "Information")
        Exit
    }

    $install_output = ./certmgr.exe -install -trace 0 -file $store_path -all -store umy -provtype 77 | Select-String "()|(Subject)"
    if (!$?) {
        [System.Windows.Forms.MessageBox]::Show("     : 0x" + [System.Convert]::ToString($lastexitcode, 16), $title, $ok, "Stop")
        Exit
    }
    Remove-Item $store_path

    $report = ""
    ForEach ($line in $($install_output -split "`r`n")) {
        $cn_pos = $line.IndexOf("CN=") + 3
        $report += "- " + $line.Substring($cn_pos) + "`r`n"
    }

    [System.Windows.Forms.MessageBox]::Show("   `r`n" + $report + "  . `r`n", $title, $ok, "Information")
}
catch {
    Write-Output "Unhandled exception..."
    Write-Output $_
}

finally {
    Write-Output "Deleting folder..."
    Remove-Item -LiteralPath $import_dir -Force -Recurse -ErrorAction SilentlyContinue
}
